/*
	File:		ODXDef.h

	Contains:	OpenDoc extension application constant definitions

	Owned by: Tantek elik

	Copyright:	 1994 - 1996 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <2>	 3/14/96	NP		1258295: Put up alert when user
									double-clicks app.
		<13>	10/26/95	T		1296574 GM:Cannot set default memory
									partition size.
		<12>	  9/8/95	eeh		1268998: prefs file name res id defined
		<11>	 8/12/95	T		1276809 Optimization: lower stack size to
									64K and default heap size to 384K
		<10>	 5/25/95	jpa		Removed CODE 7 -- CFM68K b1 doesn't
									generate one. [1241078]
		 <9>	 5/19/95	T		1249396 (was BB) OpenDoc shell should
									serialize stationery creation., 1235942
									Launching the OpenDoc pref file causes a
									1000 exception to be thrown
		 <8>	  1/6/95	jpa		Added defs for new docstub rsrcs [1196515]
		 <7>	10/10/94	T		#1191470 Need both 68K and PPC document
									stub application
		 <6>	 10/7/94	T		#1191470 Need both 68K and PPC document
									stub application
		 <5>	 8/19/94	T		#1159233 Remove references to j3di
		 <4>	 7/25/94	T		added ErrAlert constants for OpenDoc
									launcher error alert IDs
		 <3>	 7/14/94	T		added constants for document resources
		 <2>	 7/13/94	T		removed some XMP stuff
		 <1>	 6/27/94	T		first checked in
		 <0>	 6/16/94	SV		SOMverted
		 <1>	 4/18/94	T		first checked in
	
	In Progress:
		
*/

#ifndef _ODXDEF_
#define _ODXDEF_

//==============================================================================
// Constants
//==============================================================================

// ALRTs
#define kODXErrAlert 			128
#define	kODXAnonErrAlert		129
#define	kODXDoubleClickAlert	2002

// Prefs file name resource ID
#define kSHLPrefsFileNameResID		2000
#define kSHLPrefsFolderNameResID	2001

#define kPrefFileType		'pref'
// for stationery creation
#define	kEditorFileType		'shlb'
#define	kDocSeedResType		'seed'
#define	kDocSeedResID		1

// Document Stub Data					Document Stub App
// kDocStubRsrcType kDocStubRsrcID0		kcfrgRsrcType	kcfrgID
// kDocStubRsrcType kDocStubRsrcID1		kMainCFMType	kMainCFMPPCID
// kDocStubRsrcType kDocStubRsrcID2		kscszRsrcType	kscszID
// kDocStubRsrcType kDocStubRsrcID3		kMainCFMType 	kMainCFM68KID
// kDocStubRsrcType kDocStubRsrcID4		kMainCFMType 	kMainCFM68KID1
// kDocStubRsrcType kDocStubRsrcID5		kCODERsrcType	kCODEID0
// kDocStubRsrcType kDocStubRsrcID6		kCODERsrcType 	kCODEID1
// kDocStubRsrcType kDocStubRsrcID7		kCODERsrcType	kCODEID6
// kDocStubRsrcType kDocStubRsrcID8		kCODERsrcType 	kCODEID7
// kDocStubRsrcType kDocStubRsrcID9		kMainCFMType 	kSOMMemCFM68KID
// kDocStubRsrcType kDocStubRsrcID10	kMainCFMType 	kSOMMemCFMPPCID
// kSIZERsrcType 	kDocStubSIZEID		kSIZERsrcType	kSIZEID
//

// for document stub data (e.g. types & ids of the doc stub app resources when stored
// in OpenDoc & OpenDoc Shell)
#define	kSIZERsrcType		'SIZE'
#define kDocStubSIZEID		3

#define kDocStubRsrcType	'DSDT'	// for Document Stub DaTa, i.e. how these resources
									// are actually stored in OpenDoc & OpenDoc Shell
									// except for the SIZE resource which we keep as
									// a SIZE resource for the convenience of editing it
									// in Resedit.
									
#define kDocStubRsrcID0		0
#define kDocStubRsrcID1		1
#define kDocStubRsrcID2		2
#define kDocStubRsrcID3		3
#define	kDocStubRsrcID4		4
#define	kDocStubRsrcID5		5
#define	kDocStubRsrcID6		6
#define	kDocStubRsrcID7		7
#define	kDocStubRsrcID8		8
#define	kDocStubRsrcID9		9
//#define	kDocStubRsrcID10	10

// for document stub construction
#define	kMainCFMType		'rseg'  // used to be 'DSTB' for Document STuB
									// but CFM68K uses 'rseg'.
#define	kMainCFM68KID		0
#define	kMainCFM68KID1		1
#define	kMainCFMPPCID		2
#define kSOMMemCFM68KID		3
#define kSOMMemCFMPPCID		4

#define kcfrgRsrcType		'cfrg'
#define kcfrgID				0
#define	kscszRsrcType		'scsz'
#define kscszID				0

#define kDocStubStackSize	(64 * 1024)		// 64K stack size
#define	kDocStubMinHeapSize	(256 * 1024)	// 256K min app heap
#define	kDocStubDefaultSize	(384 * 1024)	// 384K default app heap

#define kCODERsrcType		'CODE'
#define kCODEID0			0
#define kCODEID1			1
#define kCODEID6			6
//#define kCODEID7			7

#define	kSIZEID				-1

// Strings for the Launcher
#define	kODXLibsFldrStrID		333
#define kODXLibsFldrStrIndex	1

#define kLaunchFailedErrStrings	301

#endif // _ODXDEF_
