/*
	File:		OpenDocX.r

	Contains:	resources for the launcher (OpenDoc)

	Owned by:	Tantek elik

	Copyright:	 1993 - 1996 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <3>	 3/14/96	NP		1258295: Definition of ALRT resource to
									display when user double-clicks app.
		 <2>	 1/15/96	JA		1269485: Added 'ictb' for splash dlog to
									change font to Geneva 10
		<15>	10/26/95	T		1296574 GM:Cannot set default memory
									partition size.
		<14>	10/25/95	T		1281409 GM: OpenDoc needs an AboutBox.
		<13>	 9/29/95	eeh		1287262: use ShellDef.h
		<12>	  9/8/95	eeh		1268998: add prefs file name 'STR '
									resource
		<11>	  6/8/95	RR		#1257260 Collapse base classes. #1214898
									StdTypes.r -> ODTypes.r
		<10>	 5/19/95	T		1155852 BB: String constants need to be
									moved to resource for Localization
		 <9>	  1/6/95	jpa		Get rsrcs from OpenDocX.res (not .rsrc) and
									DocStub.rsrc. Don't copy ckid's. [1196515]
		 <8>	12/25/94	T		1191189 Eliminate duplicate Def.h, Def.xh
									files
		 <7>	 12/5/94	jpa		Added resources for error handling. Include
									new OpenDocX.rsrc (moved many rsrcs into
									it.) [1198075]
		 <6>	 10/7/94	T		#1191470 Need both 68K and PPC document
									stub application
		 <5>	 8/19/94	T		#1159233 Remove references to j3di, all but
									the one mpwshell variable reference
		 <4>	 7/25/94	T		added Jens' Error Alert ALRT and DITL
									resources for launcher error alerts
		 <3>	 7/14/94	T		use constants for resources which are
									CHANGEd for copying into documents
		 <2>	 7/14/94	T		use full path for read statement
		 <1>	 7/13/94	T		first checked in
		 <0>	 6/27/94	SV		SOMverted
		 <5>	 4/18/94	T		#1144379 Stationery model & generation have
									been fixed to enable dragging & dropping of
									stationery onto open documents.
		 <4>	12/17/93	eeh		added line "Change 'scsz' (0) to 'J3DI'
									(3)" to allow docs to get GetAETE events
		 <3>	11/23/93	T		?
		 <2>	10/26/93	T		Change 'CODE' (2) to 'J3DI' (2);
		 <1>	10/18/93	T		first checked in

	To Do:
	In Progress:
		
*/

#ifndef _SHELLDEF_
#include "ShellDef.h"
#endif

#ifndef SOM_Module_OpenDoc_StdDefs_defined
#include "StdDefs.r"
#endif

// Resources from the ResEdit file.
// Graphical resources like ALRT, DLOG, DITL, PICT, and icons should go here.
include "OpenDocX.res" not 'ckid';

// Resources for the document stub:
include "DocStub.rsrc";


// Resources for OpenDoc the application

#define SystemSevenOrBetter 1			// we want the extended types
#define	SystemSevenOrLater	1			// Types.r uses this variable

#ifndef __TYPES_R__
#include "Types.r"
#endif

#ifndef __SYSTYPES_R__
#include "SysTypes.r"
#endif


#define kApplCrtrID		128
#define kWildCrtrID		129

resource 'ALRT' (2002, "Double-click warning") {
	{40, 40, 164, 405},
	2002,
	{	/* array: 4 elements */
		/* [1] */
		OK, visible, sound1,
		/* [2] */
		OK, visible, sound1,
		/* [3] */
		OK, visible, sound1,
		/* [4] */
		OK, visible, sound1
	},
	alertPositionMainScreen
};

resource 'DITL' (2002, "Double-click warning") {
	{	/* array DITLarray: 2 elements */
		/* [1] */
		{91, 293, 111, 351},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{11, 71, 75, 355},
		StaticText {
			disabled,
			"Double clicking on this OpenDoc icon ha"
			"s no effect. If you wish to create a new"
			" document, please use stationery -- foun"
			"d in the Stationery folder."
		}
	}
};

// SIZE -1

 RESOURCE 'SIZE' (-1) {
	reserved,
	acceptSuspendResumeEvents,
	reserved,
	canBackground,
	doesActivateOnFGSwitch,
	backgroundAndForeground,
	dontGetFrontClicks,
	acceptChildDiedEvents,	// Need to be notified by CFM if load fails
	is32BitCompatible,
	isHighLevelEventAware,
	localAndRemoteHLEvents,
	isStationeryAware,
	dontUseTextEditServices,
	reserved,
	reserved,
	reserved,
	160 * 1024,	
	80 * 1024
};


resource 'STR#' (kLaunchFailedErrStrings, purgeable) {
	{
		/* [1] */ "of an unexpected error ^3",
		/* [2] */ "of an invalid context ID",
		/* [3] */ "of an invalid connection to the library ^2",
		/* [4] */ "a symbol was not found in the library ^2",
		/* [5] */ "of a missing section in the library ^2",
		/* [6] */ "the library ^2 could not be found",
		/* [7] */ "the library name ^2 is already in use",
		/* [8] */ "the library ^2 has an unknown data format",
		/* [9] */ "the library ^2 has unresolved symbols",
		/* [10] */ "of an error ^3",
		/* [11] */ "there was not enough memory available to"
		" load the library ^2",
		/* [12] */ "there was not enough memory available to"
					" load the library ^2",
		/* [13] */ "no more context IDs are available for library ^2",
		/* [14] */ "of an order error in the init function of"
					" library ^2",
		/* [15] */ "the library ^2 is too old",
		/* [16] */ "the library ^2 is too new",
		/* [17] */ "of a circularity in the load order of library ^2",
		/* [18] */ "of an error connecting to library ^2",
		/* [19] */ "of an error connecting to library ^2",
		/* [20] */ "of an error during initialization of library ^2",
		/* [21] */ "of an internal CFM inconsistency",
		/* [22] */ "the library ^2 is corrupted: please replace it",
		/* [23] */ "of an error in the ^2 librarys initialization routine ",
		/* [24] */ "no application was found in the library ^2s 'cfrg' resource",
		/* [25] */ "the library ^2 cannot be run on this "
					"type of Macintosh; please replace it",
		/* [26] */ "the library ^2 is used invalidly"
	}
};

resource 'STR#' (kODXLibsFldrStrID, purgeable) {
	{
		"OpenDoc Libraries",
		"They're EVERYWHERE!"
	}
};

resource 'STR ' (kSHLPrefsFileNameResID, "ODPrefs file name") {
	"OpenDoc Preferences"
};

resource 'STR ' (kSHLPrefsFolderNameResID, "ODPrefs folder name") {
	"OpenDoc Prefs"
};

data 'ictb' (kODSplashDlogID) {
	$"0000 0000"	/* User item is unaffected */
	$"0005 0008"	/* Change font + size of text item */
	
	$"0001"			/* Use application font */
	$"0000"			/* Plain style [ignored] */
	$"000A"			/* 10pt size */
	$"0000"			/* Red component [ignored] */
	$"0000"			/* Blue component [ignored] */
	$"0000"			/* Green component [ignored] */
	$"0000"			/* BG Red component [ignored] */
	$"0000"			/* BG Blue component [ignored] */
	$"0000"			/* BG Green component [ignored] */
	$"0001"			/* srcOr mode [ignored] */
};
	