/*
	File:		PolyShpe.h

	Contains:	PolygonShape class, private to ODShape.

	Written by:	Jens Alfke

	Copyright:	1993-94 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <4>	 12/5/94	jpa		Code review cleanup [1203923]. Also
									simplify before clipping [1196018]
		 <3>	10/24/94	jpa		Added fSimple flag [1191192]
		 <2>	  8/8/94	jpa		Added Outset method [1178690]
		 <1>	 6/15/94	jpa		first checked in
							--------Moved to SOM project and rearranged-----
		 <7>	  5/9/94	jpa		Enable polygon clipper and GX support
									[1162090]
		 <6>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		 <5>	 2/17/94	JA		Declare XMPPolygon as struct, not class.
		 <4>	 2/16/94	JA		Include new AltPoint.h.
		 <3>	 2/10/94	JA		Stop using ptrs to member fns.
		 <2>	  2/9/94	JA		Renamed from ShapePrv.h --> ShapePvM.h
		 <6>	  2/7/94	JA		Further tigerings.
		 <5>	  2/3/94	JA		Tiger Team Makeover!
		 <4>	 1/31/94	JA		XMPRealShape no longer derives from
									XMPAbsShape. Other API improvements.
		 <3>	11/24/93	VL		Rolled back changes as ASLM build breaks
									the THINK build.
		 <2>	11/23/93	VL		Made this work with ASLM.
		 <1>	11/23/93	JA		first checked in

	In Progress:
*/


#ifndef _POLYSHPE_
#define _POLYSHPE_

#ifndef _ODTYPES_
#include "ODTypes.h"
#endif

#ifndef _PLFMDEF_
#include "PlfmDef.h"
#endif

#ifndef _REALSHPE_
#include "RealShpe.h"		/* Base class*/
#endif


//==============================================================================
// PolyShape
//==============================================================================


class PolygonShape :public RealShape {
	public:
	PolygonShape( ODGeometryMode );
	virtual ~PolygonShape( );

	ODVMethod ODSize		Purge( ODSize );
	
	ODVMethod void			GetBoundingBox( ODRect *bounds );
	ODVMethod RealShape*	SetRectangle( const ODRect *rect );
	ODVMethod void			CopyPolygon( ODPolygon& );
	ODVMethod RealShape*	SetPolygon( const ODPolygon& );
	ODVMethod void			Simplify( );			// Call after SetPolygon if poly isn't simple

	ODVMethod ODBoolean		IsSameAs( RealShape* compareShape );
	ODVMethod ODBoolean		IsEmpty( );
	ODVMethod ODBoolean		ContainsPoint(ODPoint point);
	ODVMethod ODBoolean		IsRectangular( );
	
	ODVMethod RealShape*	Copy( );	// Really returns an ODPolygonShape*

	ODVMethod RealShape*	Transform( Environment*, ODTransform* );
	ODVMethod RealShape*	Outset(ODCoordinate distance);
	ODVMethod RealShape*	Subtract(RealShape* diffShape);
	ODVMethod RealShape*	Intersect(RealShape* sectShape);
	ODVMethod RealShape*	Union(RealShape* sectShape);
	
	ODVMethod RealShape*	Combine( ODShapeOp, RealShape* );

	protected:
	ODVMethod void			InitQDRegion( );
	ODVMethod gxShape		CopyGXShape( );

	private:
			
	ODPolygon	fPolygon;
	ODRect		fBounds;
	ODBoolean	fBoundsValid;
};


#endif /*_POLYSHPE_*/