/*
	File:		DfltAcs.h

	Contains:	Implementation of Mac specific default accessors and handlers.

	Owned by:	Eric House

	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<14>	 6/20/95	NP		1260389: Fix DrawPart's count proc
		<13>	  6/7/95	eeh		1251403: add ASSERT_FRAME_MATCHES_PART
		<12>	 5/18/95	eeh		1250061: add CountEmbeddedFrames
		<11>	 4/14/95	NP		1239651: Remove shared constants from here.
		<10>	  4/7/95	eeh		1236842: added TrySwapToRootPart
		 <9>	 3/24/95	eeh		1232249: remove frame from
									DEFAULT_ACCESSOR_PARAMS
		 <8>	 3/13/95	NP		1223764: CreateSwapToken should take an
									ODFrame* parameter. Add frame param to
									DEFAULT_ACCESSOR_PARAMS.
		 <7>	  3/2/95	eeh		1214783: finish work on lists
		 <6>	 2/22/95	eeh		1222901: add StandardPartToken
		 <5>	 2/21/95	eeh		1214783: Accessor/handler API changes
		 <4>	  2/3/95	eeh		1217393: mod. params to use new ODDesc etc.
		 <3>	 1/27/95	TJ		NP: Added forward class declarations.
		 <2>	 1/25/95	eeh		1214783: more implementation
		 <1>	  1/18/95	eeh		first checked in

	To Do:
*/

#ifndef _DFLTACS_
#define _DFLTACS_


#ifndef _CNTXTOSL_
#include "CntxtOSL.h"
#endif

#if ODDebug
void ASSERT_FRAME_MATCHES_PART( Environment *ev, ODFrame* frame, ODPart* part );
#else
#define ASSERT_FRAME_MATCHES_PART( ev, frame, part )
#endif

class ODDesc;
class ODOSLToken;
class ODAppleEvent;
class DefaultAccessorSI;
class ODSession;


#define DEFAULT_ACCESSOR_PARAMS	Environment		*ev,					\
								ODPart*			scriptlessPart,			\
								DescType		desiredClass,			\
								OSLToken*		container,				\
								DescType		containerClass,			\
								DescType		form,					\
								ODDesc*			selectionData,			\
								ODOSLToken*		value,					\
								ODSession*		session					\

//								ODFrame*		frame					\
								
typedef ODError (*PartAccessorProc)(DEFAULT_ACCESSOR_PARAMS);

ODError GetWildcardFromPart(DEFAULT_ACCESSOR_PARAMS);

ODError TrySwapToRootPart(DEFAULT_ACCESSOR_PARAMS);

ODError GetPartFromNULL(DEFAULT_ACCESSOR_PARAMS);

ODError GetPropFromNULL(DEFAULT_ACCESSOR_PARAMS);

ODError GetWildcardFromList(DEFAULT_ACCESSOR_PARAMS);


typedef ODError (*EventHandlerProc)( Environment *ev,
		ODPart* thePart, ODAppleEvent* message,
		ODAppleEvent* reply, ODSession* session);

ODError HandleGetData(	Environment *ev,
						ODPart* thePart,
						ODAppleEvent* message,
						ODAppleEvent* reply,
						ODSession* session);

ODError HandleSetData(	Environment *ev,
						ODPart* thePart,
						ODAppleEvent* message,
						ODAppleEvent* reply,
						ODSession* session);

#endif

