/*
	File:		DrafPriv.h

	Contains:	Private class used by CMDraft.

	Owned by:	Vincent Lo

	Copyright:	 1993 - 1995 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <8>	 6/16/95	VL		1244940: Removed fEV from PreserveFocus
									class.
		 <7>	 6/13/95	VL		1241352: Moved PreserveFocus from
									CMDraft.cpp.
		 <6>	 4/25/95	DM		1172791: Replace AEHashTable with
									OpenHashTable
		 <5>	 8/11/94	CC		Added #define _DRAFPRIV_.
		 <4>	 6/20/94	CC		ODMemoryHeap* changed to ODMemoryHeapID.
		 <3>	 6/15/94	RR		ODHeap -> ODMemoryHeap
		 <2>	 5/27/94	VL		Changed ODAEHashTable to AEHashTable.
		 <1>	 5/27/94	VL		first checked in

	To Do:
*/

#ifndef _DRAFPRIV_
#define _DRAFPRIV_

#ifndef _ODTYPES_
#include "ODTypes.h"
#endif

#ifndef _PLFMDEF_
#include "PlfmDef.h"
#endif

#ifndef _ODMEMORY_
#include "ODMemory.h"
#endif

class OpenHashTable;
class ODStorageUnit;
class ODStorageUnitCursor;

class IDList {

public:

	IDList();
	ODVMethod	~IDList();
	
	ODMethod void Initialize();
	
	ODMethod void	Add(ODID id, void* object);
	ODMethod ODID Add(void* object);
	ODMethod void 	Remove(ODID id);

	ODMethod void*	Get(ODID id);
	ODMethod ODID	GetID(void* object);
	
	ODMethod ODBoolean	Exists(ODID id);
	ODMethod ODBoolean	ObjectExists(void* object);

private:

	ODMemoryHeapID			GetHeap();
	
	OpenHashTable*	fIDToObj;
	OpenHashTable*	fObjToID;
	ODID			fCurrentID;
	ODMemoryHeapID			fHeap;
};

class PreserveFocus
{

public:

	PreserveFocus(Environment* ev, ODStorageUnit* su);
	~PreserveFocus();
		
private:

	ODStorageUnit* fSU;
	ODStorageUnitCursor* fSUCursor;
};

#endif	// _DRAFPRIV_