/*
	File:		SIHelpMac.h

	Contains:	Definition of an abstract helper class used by
				ODCPlusSemanticInterface

	Owned by:	Nick Pilch

	Copyright:	 1993 - 1995 by Apple Computer, Inc., all rights reserved.

	
*/

#ifndef _SIHLPABS_
#define _SIHLPABS_


#ifndef _ODTYPES_
#include "ODTypes.h"
#endif

#ifndef SOM_ODDesc_xh
#include "ODDesc.xh"
#endif

//==============================================================================
// Classes defined in this interface
//==============================================================================

class	SIHelperAbs;

//==============================================================================
// Classes used by this interface
//==============================================================================

class	ODPart;
class	ODAppleEvent;
class	ODOSLToken;

//==============================================================================
// SIHelperAbs
//==============================================================================

#define ODVMethod	virtual

class SIHelperAbs
{
  public:

	ODVMethod ~SIHelperAbs() {}
	
	ODVMethod ODError   CallEventHandler(
			ODPart* thePart,
			ODAppleEvent* theODAppleEvent,
			ODAppleEvent* reply) = 0;
	ODVMethod ODError   CallPredispatchProc(
			ODPart* thePart,
			ODAppleEvent* theODAppleEvent,
			ODAppleEvent* reply) = 0;
	ODVMethod ODError   CallCoercionHandler(
			ODPart* thePart,
			ODDesc* theODDesc,
			ODDescType toType,
			ODDesc* result) = 0;
	ODVMethod ODError   CallObjectAccessor(
			ODPart* thePart,
			ODDescType desiredClass,
			ODOSLToken* container,
			ODDescType containerClass,
			ODDescType form,
			ODDesc* selectionData,
			ODOSLToken* value) = 0;
	ODVMethod ODError   CallCompareProc(
		ODPart* thePart,
		ODDescType oper,
		ODOSLToken* obj1,
		ODOSLToken* obj2,
		ODBoolean* result) = 0;
	ODVMethod ODError   CallCountProc(
		ODPart* thePart,
		ODDescType desiredType,
		ODDescType containerClass,
		ODOSLToken* container,
		ODSLong* result) = 0;
	ODVMethod ODError   CallDisposeTokenProc(
		ODPart* thePart,
		ODOSLToken* unneededToken) = 0;
	ODVMethod ODError   CallGetErrDescProc(
		ODPart* thePart,
		ODDesc** errDesc) = 0;
	ODVMethod ODError   CallGetMarkTokenProc(
		ODPart* thePart,
		ODOSLToken* dContainerToken,
		ODDescType containerClass,
		ODOSLToken* result) = 0;
	ODVMethod ODError   CallMarkProc(
		ODPart* thePart,
		ODOSLToken* dToken,
		ODOSLToken* markToken,
		ODSLong index) = 0;
	ODVMethod ODError   CallAdjustMarksProc(
		ODPart* thePart,
		ODSLong newStart,
		ODSLong newStop,
		ODOSLToken* markToken) = 0;
};


#endif // _SIHLPABS_